@@ -4,9 +4,10 @@ use 5.006001;
 use strict;
 use warnings;
 
+use File::Spec;
 use File::Find;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 our $basedir = undef;
 our @results = ();
@@ -14,7 +15,7 @@ our $prune = 0;
 
 our @ISA = qw(Exporter);
 
-our @EXPORT = qw(findsubmod findallmod usesub useall);
+our @EXPORT = qw(findsubmod findallmod usesub useall setmoduledirs);
 
 =head1 NAME
 
@@ -35,6 +36,9 @@ Module::Find - Find and use installed modules in a (sub)category
 
   # find anything in the CGI/ directory
   @found = findallmod CGI;
+  
+  # set your own search dirs (uses @INC otherwise)
+  setmoduledirs(@INC, @plugindirs, $appdir);
 
 =head1 DESCRIPTION
 
@@ -50,6 +54,19 @@ make sure you modify C<@INC> before you call the Module::Find functions.
 
 =over
 
+=item C<setmoduledirs(@directories)>
+
+Sets the directories to be searched for modules. If not set, Module::Find
+will use @INC. If you use this function, @INC will I<not> be included
+automatically, so add it if you want it. Set to undef to revert to
+default behaviour.
+
+=cut
+
+sub setmoduledirs {
+    return @Module::Find::ModuleDirs = @_;
+}
+
 =item C<@found = findsubmod Module::Category>
 
 Returns modules found in the Module/Category subdirectories of your perl 
@@ -91,8 +108,8 @@ sub usesub(*) {
 	
 	my @r = _find($_[0]);
 	
-	foreach (@r) {
-		eval " require $_; import $_ ; ";
+	foreach my $m (@r) {
+		eval " require $m; import $m ; ";
 		die $@ if $@;
 	}
 	
@@ -112,8 +129,8 @@ sub useall(*) {
 	
 	my @r = _find($_[0]);
 	
-	foreach (@r) {
-		eval " require $_; import $_; ";
+	foreach my $m (@r) {
+		eval " require $m; import $m; ";
 		die $@ if $@;
 	}
 	
@@ -123,7 +140,7 @@ sub useall(*) {
 # 'wanted' functions for find()
 # you know, this would be a nice application for currying...
 sub _wanted {
-    (my $name = $_) =~ s|^$basedir/?||;
+    my $name = File::Spec->abs2rel($_, $basedir);
     return unless $name;
 
     if (-d && $prune) {
@@ -133,8 +150,8 @@ sub _wanted {
 
     return unless /\.pm$/ && -r;
 
-    $name =~ s|/|::|g;
     $name =~ s|\.pm$||;
+    $name = join('::', File::Spec->splitdir($name));
 
     push @results, $name;
 }
@@ -146,13 +163,19 @@ sub _find(*) {
     my ($category) = @_;
     return undef unless defined $category;
 
-    (my $dir = $category) =~ s|::|/|g;
+    my $dir = File::Spec->catdir(split(/::/, $category));
 
-    my @dirs = map "$_/$dir", @INC;
+    my @dirs;
+    if (defined @Module::Find::ModuleDirs) {
+        @dirs = map { File::Spec->catdir($_, $dir) }
+            @Module::Find::ModuleDirs;
+    } else {
+        @dirs = map { File::Spec->catdir($_, $dir) } @INC;
+    }
     @results = ();
 
     foreach $basedir (@dirs) {
-    	next unless -d $basedir;
+        	next unless -d $basedir;
     	
         find({wanted   => \&_wanted,
               no_chdir => 1}, $basedir);
@@ -177,6 +200,17 @@ Original version; created by h2xs 1.22
 Added test modules that were left out in the first version. Thanks to
 Stuart Johnston for alerting me to this.
 
+=item 0.03, 2004-06-18
+
+Fixed a bug (non-localized $_) by declaring a loop variable in use functions.
+Thanks to Stuart Johnston for alerting me to this and providing a fix.
+
+Fixed non-platform compatibility by using File::Spec.
+Thanks to brian d foy.
+
+Added setmoduledirs and updated tests. Idea shamelessly stolen from
+...errm... inspired by brian d foy.
+
 =back
 
 =head1 SEE ALSO
@@ -189,7 +223,7 @@ Christian Renz, E<lt>crenz@web42.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2004 by Christian Renz <crenz@web42.com>
+Copyright 2004 by Christian Renz <crenz@web42.com>. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself. 
